<?php

class mypos_virtual {

    private $_defaultMethodProperties = array(
        'IPCmethod' => null,
        'KeyIndex' => null,
        'IPCVersion' => null,
        'IPCLanguage' => null,
        'WalletNumber' => null,
        'SID' => null
        //'Signature' is added last, because of the API requirements
    );

    public function __construct() {
        /**
         * Sets initial member variables and module status (enabled/disabled).
         * Define the api_url variable which is your gateway url (that will recieve the POST data).
         * Include any static variables you may require in your code here.
         */
        global $HTTP_GET_VARS, $PHP_SELF, $order, $currency;

        $this->signature = 'mypos_eu|mypos_eu|1.0|2.3';
        $this->api_version = '1.4';

        $this->code = 'mypos_virtual';

        if (MODULE_PAYMENT_MYPOS_VIRTUAL_PAYMENT_METHOD == 'All' && $order->info['currency'] == 'EUR') {
            $description = MODULE_PAYMENT_MYPOS_VIRTUAL_TEXT_PUBLIC_DESCRIPTION_ALL;
        } elseif (MODULE_PAYMENT_MYPOS_VIRTUAL_PAYMENT_METHOD == 'iDeal' && $order->info['currency'] == 'EUR') {
            $description = MODULE_PAYMENT_MYPOS_VIRTUAL_TEXT_PUBLIC_DESCRIPTION_IDEAL;
        } else {
            $description = MODULE_PAYMENT_MYPOS_VIRTUAL_TEXT_PUBLIC_DESCRIPTION_CARD_PAYMENT;
        }

        $this->title = MODULE_PAYMENT_MYPOS_VIRTUAL_TEXT_TITLE;
        $this->public_title = MODULE_PAYMENT_MYPOS_VIRTUAL_TEXT_PUBLIC_TITLE . ' ' . $description;;
        $this->description = MODULE_PAYMENT_MYPOS_VIRTUAL_TEXT_DESCRIPTION;

        $this->sort_order = 0;
        if (defined('MODULE_PAYMENT_MYPOS_VIRTUAL_SORT_ORDER')) {
            $this->sort_order = MODULE_PAYMENT_MYPOS_VIRTUAL_SORT_ORDER;
        }

        $this->enabled = false;
        if (defined('MODULE_PAYMENT_MYPOS_VIRTUAL_STATUS') && (MODULE_PAYMENT_MYPOS_VIRTUAL_STATUS == 'True')
            &&  (MODULE_PAYMENT_MYPOS_VIRTUAL_PAYMENT_METHOD != 'iDeal'
                || ( MODULE_PAYMENT_MYPOS_VIRTUAL_PAYMENT_METHOD == 'iDeal' && $currency == 'EUR'))) {
            $this->enabled = true;
        }

        $this->order_status = 0;
        if (defined('MODULE_PAYMENT_MYPOS_VIRTUAL_ORDER_STATUS_ID') && ((int) MODULE_PAYMENT_MYPOS_VIRTUAL_ORDER_STATUS_ID > 0)) {
            $this->order_status = (int) MODULE_PAYMENT_MYPOS_VIRTUAL_ORDER_STATUS_ID;
        }

        $configurationPack = json_decode(base64_decode(MODULE_PAYMENT_MYPOS_VIRTUAL_CONFIGURATION_PACK), true);

        $this->private_key = !empty($configurationPack['pk']) ? $configurationPack['pk'] : MODULE_PAYMENT_MYPOS_VIRTUAL_PRIVATE_KEY;
        $this->public_certificate = !empty($configurationPack['pc']) ? $configurationPack['pc'] : MODULE_PAYMENT_MYPOS_VIRTUAL_PUBLIC_CERTIFICATE;

        $mypos_errors = array();

        if ((!tep_not_null(MODULE_PAYMENT_MYPOS_VIRTUAL_SITE_ID) ||
                !tep_not_null(MODULE_PAYMENT_MYPOS_VIRTUAL_WALLET_NR) ||
                !tep_not_null(MODULE_PAYMENT_MYPOS_VIRTUAL_PRIVATE_KEY) ||
                !tep_not_null(MODULE_PAYMENT_MYPOS_VIRTUAL_PUBLIC_CERTIFICATE) ||
                !tep_not_null(MODULE_PAYMENT_MYPOS_VIRTUAL_KEYINDEX) ||
                !tep_not_null(MODULE_PAYMENT_MYPOS_VIRTUAL_DEVELOPER_URL) ||
                !tep_not_null(MODULE_PAYMENT_MYPOS_VIRTUAL_PRODUCTION_URL)
            ) &&
            !tep_not_null(MODULE_PAYMENT_MYPOS_VIRTUAL_CONFIGURATION_PACK)
        ) {
            $mypos_errors[] = MODULE_PAYMENT_MYPOS_VIRTUAL_ERROR_ADMIN_CONFIGURATION;
        }

        if (!empty($mypos_errors)) {
            $html_errors = array();
            foreach ($mypos_errors as $err) {
                $html_errors[] = '<div class="secWarning">' . $err . '</div>';
            }

            $this->description = implode('<br />', $html_errors) . '<br />' . $this->description;

            $this->enabled = false;
        }

        if ($this->enabled) {
            if (isset($order) && is_object($order)) {
                $this->update_status();
            }
        }

        $this->_defaultMethodProperties['KeyIndex'] = !empty($configurationPack['idx']) ? $configurationPack['idx'] : MODULE_PAYMENT_MYPOS_VIRTUAL_KEYINDEX;
        $this->_defaultMethodProperties['IPCVersion'] = $this->api_version;
        $this->_defaultMethodProperties['IPCLanguage'] = 'EN';
        $this->_defaultMethodProperties['WalletNumber'] = !empty($configurationPack['cn']) ? $configurationPack['cn'] : MODULE_PAYMENT_MYPOS_VIRTUAL_WALLET_NR;
        $this->_defaultMethodProperties['SID'] = !empty($configurationPack['sid']) ? $configurationPack['sid'] : MODULE_PAYMENT_MYPOS_VIRTUAL_SITE_ID;
        $this->_defaultMethodProperties['Source'] = 'sc_oscommerce 1.3.1';

        $this->form_action_url = MODULE_PAYMENT_MYPOS_VIRTUAL_TEST_MODE == 'True' ? MODULE_PAYMENT_MYPOS_VIRTUAL_DEVELOPER_URL : MODULE_PAYMENT_MYPOS_VIRTUAL_PRODUCTION_URL;

        //.com migration
        if (empty(MODULE_PAYMENT_MYPOS_VIRTUAL_DEVELOPER_URL) || false !== stripos((parse_url(MODULE_PAYMENT_MYPOS_VIRTUAL_DEVELOPER_URL)['host']), 'mypos.eu')) {
            $newTestUrl = 'https://www.mypos.com/vmp/checkout-test';
            $this->editConfig('MODULE_PAYMENT_MYPOS_VIRTUAL_DEVELOPER_URL', array(
                'value' => $newTestUrl
            ));

            if (MODULE_PAYMENT_MYPOS_VIRTUAL_TEST_MODE == 'True') {
                $this->form_action_url = $newTestUrl;
            }

        }

        if (empty(MODULE_PAYMENT_MYPOS_VIRTUAL_PRODUCTION_URL) || false !== stripos((parse_url(MODULE_PAYMENT_MYPOS_VIRTUAL_PRODUCTION_URL)['host']), 'mypos.eu')) {
            $newProdUrl = 'https://www.mypos.com/vmp/checkout';
            $this->editConfig('MODULE_PAYMENT_MYPOS_VIRTUAL_PRODUCTION_URL', array(
                'value' => $newProdUrl,
            ));

            if (MODULE_PAYMENT_MYPOS_VIRTUAL_TEST_MODE != 'True') {
                $this->form_action_url = $newTestUrl;
            }
        }
        //end .com migration
    }

    public function update_status() {
        /**
         * Called by module's class constructor, checkout_confirmation.php, checkout_process.php
         */
        global $order;
    }

    public function checkout_initialization_method() {
        return false;
    }

    public function javascript_validation() {
        /**
         * Here you may define client side javascript that will verify any input fields you use in the payment method selection page. Refer to standard cc module as reference (cc.php).
         * Called by checkout_payment.php
         */
        return false;
    }

    public function selection() {
        /**
         * This function outputs the payment method title/text and if required, the input fields.
         * Called by checkout_payment.php
         */
        return array(
            'id' => $this->code,
            'module' => $this->public_title
        );
    }

    public function pre_confirmation_check() {
        global $cartID, $cart, $order;

        if (empty($cart->cartID)) {
            $cartID = $cart->cartID = $cart->generate_cart_id();
        }

        if (!tep_session_is_registered('cartID')) {
            tep_session_register('cartID');
        }

        $order->info['payment_method_raw'] = $order->info['payment_method'];

        if (MODULE_PAYMENT_MYPOS_VIRTUAL_PAYMENT_METHOD == 'All' && $order->info['currency'] == 'EUR') {
            $order->info['payment_method'] = '<img src="includes/modules/payment/mypos/card_schemes_ideal_no_bg.png" border="0" alt="myPOS Logo" />';
        } elseif (MODULE_PAYMENT_MYPOS_VIRTUAL_PAYMENT_METHOD == 'iDeal' && $order->info['currency'] == 'EUR') {
            $order->info['payment_method'] = '<img src="includes/modules/payment/mypos/mypos_ideal_no_bg.png" border="0" alt="myPOS Logo" />';
        } else {
            $order->info['payment_method'] = '<img src="includes/modules/payment/mypos/card_schemes_no_bg.png" border="0" alt="myPOS Logo" />';
        }
    }

    public function confirmation() {
        /**
         * Implement any checks or processing on the order information before proceeding to payment confirmation. You most probably don't need to implement anything here.
         * Called by checkout_confirmation.php
         */

        global $cartID, $cart_myPOS_Virtual_ID, $customer_id, $languages_id, $order, $order_total_modules;

        if (tep_session_is_registered('cartID')) {
            $insert_order = false;

            if (tep_session_is_registered('cart_myPOS_Virtual_ID')) {
                $order_id = substr($cart_myPOS_Virtual_ID, strpos($cart_myPOS_Virtual_ID, '-')+1);

                $curr_check = tep_db_query("select currency from " . TABLE_ORDERS . " where orders_id = '" . (int)$order_id . "'");
                $curr = tep_db_fetch_array($curr_check);

                if ( ($curr['currency'] != $order->info['currency']) || ($cartID != substr($cart_myPOS_Virtual_ID, 0, strlen($cartID))) ) {
                    $check_query = tep_db_query('select orders_id from ' . TABLE_ORDERS_STATUS_HISTORY . ' where orders_id = "' . (int)$order_id . '" limit 1');

                    if (tep_db_num_rows($check_query) < 1) {
                        tep_db_query('delete from ' . TABLE_ORDERS . ' where orders_id = "' . (int)$order_id . '"');
                        tep_db_query('delete from ' . TABLE_ORDERS_TOTAL . ' where orders_id = "' . (int)$order_id . '"');
                        tep_db_query('delete from ' . TABLE_ORDERS_STATUS_HISTORY . ' where orders_id = "' . (int)$order_id . '"');
                        tep_db_query('delete from ' . TABLE_ORDERS_PRODUCTS . ' where orders_id = "' . (int)$order_id . '"');
                        tep_db_query('delete from ' . TABLE_ORDERS_PRODUCTS_ATTRIBUTES . ' where orders_id = "' . (int)$order_id . '"');
                        tep_db_query('delete from ' . TABLE_ORDERS_PRODUCTS_DOWNLOAD . ' where orders_id = "' . (int)$order_id . '"');
                    }

                    $insert_order = true;
                }
            } else {
                $insert_order = true;
            }

            if ($insert_order == true) {
                $order_totals = array();
                if (is_array($order_total_modules->modules)) {
                    foreach ($order_total_modules->modules as $value) {
                        $class = substr($value, 0, strrpos($value, '.'));
                        if ($GLOBALS[$class]->enabled) {
                            for ($i=0, $n=sizeof($GLOBALS[$class]->output); $i<$n; $i++) {
                                if (tep_not_null($GLOBALS[$class]->output[$i]['title']) && tep_not_null($GLOBALS[$class]->output[$i]['text'])) {
                                    $order_totals[] = array('code' => $GLOBALS[$class]->code,
                                        'title' => $GLOBALS[$class]->output[$i]['title'],
                                        'text' => $GLOBALS[$class]->output[$i]['text'],
                                        'value' => $GLOBALS[$class]->output[$i]['value'],
                                        'sort_order' => $GLOBALS[$class]->sort_order);
                                }
                            }
                        }
                    }
                }

                if ( isset($order->info['payment_method_raw']) ) {
                    $order->info['payment_method'] = $order->info['payment_method_raw'];
                    unset($order->info['payment_method_raw']);
                }

                $sql_data_array = array('customers_id' => $customer_id,
                    'customers_name' => $order->customer['firstname'] . ' ' . $order->customer['lastname'],
                    'customers_company' => $order->customer['company'],
                    'customers_street_address' => $order->customer['street_address'],
                    'customers_suburb' => $order->customer['suburb'],
                    'customers_city' => $order->customer['city'],
                    'customers_postcode' => $order->customer['postcode'],
                    'customers_state' => $order->customer['state'],
                    'customers_country' => $order->customer['country']['title'],
                    'customers_telephone' => $order->customer['telephone'],
                    'customers_email_address' => $order->customer['email_address'],
                    'customers_address_format_id' => $order->customer['format_id'],
                    'delivery_name' => $order->delivery['firstname'] . ' ' . $order->delivery['lastname'],
                    'delivery_company' => $order->delivery['company'],
                    'delivery_street_address' => $order->delivery['street_address'],
                    'delivery_suburb' => $order->delivery['suburb'],
                    'delivery_city' => $order->delivery['city'],
                    'delivery_postcode' => $order->delivery['postcode'],
                    'delivery_state' => $order->delivery['state'],
                    'delivery_country' => $order->delivery['country']['title'],
                    'delivery_address_format_id' => $order->delivery['format_id'],
                    'billing_name' => $order->billing['firstname'] . ' ' . $order->billing['lastname'],
                    'billing_company' => $order->billing['company'],
                    'billing_street_address' => $order->billing['street_address'],
                    'billing_suburb' => $order->billing['suburb'],
                    'billing_city' => $order->billing['city'],
                    'billing_postcode' => $order->billing['postcode'],
                    'billing_state' => $order->billing['state'],
                    'billing_country' => $order->billing['country']['title'],
                    'billing_address_format_id' => $order->billing['format_id'],
                    'payment_method' => $order->info['payment_method'],
                    'cc_type' => $order->info['cc_type'],
                    'cc_owner' => $order->info['cc_owner'],
                    'cc_number' => $order->info['cc_number'],
                    'cc_expires' => $order->info['cc_expires'],
                    'date_purchased' => 'now()',
                    'orders_status' => $order->info['order_status'],
                    'currency' => $order->info['currency'],
                    'currency_value' => $order->info['currency_value']);

                tep_db_perform(TABLE_ORDERS, $sql_data_array);

                $insert_id = tep_db_insert_id();

                for ($i=0, $n=sizeof($order_totals); $i<$n; $i++) {
                    $sql_data_array = array('orders_id' => $insert_id,
                        'title' => $order_totals[$i]['title'],
                        'text' => $order_totals[$i]['text'],
                        'value' => $order_totals[$i]['value'],
                        'class' => $order_totals[$i]['code'],
                        'sort_order' => $order_totals[$i]['sort_order']);

                    tep_db_perform(TABLE_ORDERS_TOTAL, $sql_data_array);
                }

                for ($i=0, $n=sizeof($order->products); $i<$n; $i++) {
                    $sql_data_array = array('orders_id' => $insert_id,
                        'products_id' => tep_get_prid($order->products[$i]['id']),
                        'products_model' => $order->products[$i]['model'],
                        'products_name' => $order->products[$i]['name'],
                        'products_price' => $order->products[$i]['price'],
                        'final_price' => $order->products[$i]['final_price'],
                        'products_tax' => $order->products[$i]['tax'],
                        'products_quantity' => $order->products[$i]['qty']);

                    tep_db_perform(TABLE_ORDERS_PRODUCTS, $sql_data_array);

                    $order_products_id = tep_db_insert_id();

                    $attributes_exist = '0';
                    if (isset($order->products[$i]['attributes'])) {
                        $attributes_exist = '1';
                        for ($j=0, $n2=sizeof($order->products[$i]['attributes']); $j<$n2; $j++) {
                            if (DOWNLOAD_ENABLED == 'true') {
                                $attributes_query = "select popt.products_options_name, poval.products_options_values_name, pa.options_values_price, pa.price_prefix, pad.products_attributes_maxdays, pad.products_attributes_maxcount , pad.products_attributes_filename
                                       from " . TABLE_PRODUCTS_OPTIONS . " popt, " . TABLE_PRODUCTS_OPTIONS_VALUES . " poval, " . TABLE_PRODUCTS_ATTRIBUTES . " pa
                                       left join " . TABLE_PRODUCTS_ATTRIBUTES_DOWNLOAD . " pad
                                       on pa.products_attributes_id=pad.products_attributes_id
                                       where pa.products_id = '" . $order->products[$i]['id'] . "'
                                       and pa.options_id = '" . $order->products[$i]['attributes'][$j]['option_id'] . "'
                                       and pa.options_id = popt.products_options_id
                                       and pa.options_values_id = '" . $order->products[$i]['attributes'][$j]['value_id'] . "'
                                       and pa.options_values_id = poval.products_options_values_id
                                       and popt.language_id = '" . $languages_id . "'
                                       and poval.language_id = '" . $languages_id . "'";
                                $attributes = tep_db_query($attributes_query);
                            } else {
                                $attributes = tep_db_query("select popt.products_options_name, poval.products_options_values_name, pa.options_values_price, pa.price_prefix from " . TABLE_PRODUCTS_OPTIONS . " popt, " . TABLE_PRODUCTS_OPTIONS_VALUES . " poval, " . TABLE_PRODUCTS_ATTRIBUTES . " pa where pa.products_id = '" . $order->products[$i]['id'] . "' and pa.options_id = '" . $order->products[$i]['attributes'][$j]['option_id'] . "' and pa.options_id = popt.products_options_id and pa.options_values_id = '" . $order->products[$i]['attributes'][$j]['value_id'] . "' and pa.options_values_id = poval.products_options_values_id and popt.language_id = '" . $languages_id . "' and poval.language_id = '" . $languages_id . "'");
                            }
                            $attributes_values = tep_db_fetch_array($attributes);

                            $sql_data_array = array('orders_id' => $insert_id,
                                'orders_products_id' => $order_products_id,
                                'products_options' => $attributes_values['products_options_name'],
                                'products_options_values' => $attributes_values['products_options_values_name'],
                                'options_values_price' => $attributes_values['options_values_price'],
                                'price_prefix' => $attributes_values['price_prefix']);

                            tep_db_perform(TABLE_ORDERS_PRODUCTS_ATTRIBUTES, $sql_data_array);

                            if ((DOWNLOAD_ENABLED == 'true') && isset($attributes_values['products_attributes_filename']) && tep_not_null($attributes_values['products_attributes_filename'])) {
                                $sql_data_array = array('orders_id' => $insert_id,
                                    'orders_products_id' => $order_products_id,
                                    'orders_products_filename' => $attributes_values['products_attributes_filename'],
                                    'download_maxdays' => $attributes_values['products_attributes_maxdays'],
                                    'download_count' => $attributes_values['products_attributes_maxcount']);

                                tep_db_perform(TABLE_ORDERS_PRODUCTS_DOWNLOAD, $sql_data_array);
                            }
                        }
                    }
                }

                $cart_myPOS_Virtual_ID = $cartID . '-' . $insert_id;
                tep_session_register('cart_myPOS_Virtual_ID');
            }
        }

        return false;
    }

    public function process_button() {
        /**
         * Outputs the html form hidden elements sent as POST data to the payment gateway.
         * Called by checkout_confirmation.php
         */

        global $cart, $order, $cart_myPOS_Virtual_ID;
        $order_id =  substr($cart_myPOS_Virtual_ID, strpos($cart_myPOS_Virtual_ID, '-')+1);
        $testPrefix = MODULE_PAYMENT_MYPOS_VIRTUAL_TEST_MODE == 'True' ? MODULE_PAYMENT_MYPOS_VIRTUAL_TEST_PREFIX : '';
        $params = $this->_defaultMethodProperties;
        $query = "SELECT * FROM " . TABLE_LANGUAGES . " WHERE languages_id = " . $_SESSION['languages_id'];
        $result = tep_db_fetch_array(tep_db_query($query));
        $params['IPCLanguage'] = $result['code'];

        $params['IPCmethod'] = 'IPCPurchase';

        $params['Amount'] = number_format($order->info['total'], 2, '.', '');
        $params['Currency'] = $order->info['currency'];
        $params['OrderID'] = $testPrefix . $order_id;
        $params['URL_OK'] = tep_href_link(FILENAME_CHECKOUT_PROCESS, '', 'SSL');
        $params['URL_CANCEL'] = tep_href_link(FILENAME_CHECKOUT_PAYMENT, '', 'SSL');
        $params['URL_Notify'] = tep_href_link('ext/modules/payment/mypos_virtual/mypos_virtual.php', '', 'SSL', false, false);
        $params['CustomerIP'] = $_SERVER['REMOTE_ADDR'];
        $params['CustomerEmail'] = $order->customer['email_address'];
        $params['CustomerFirstNames'] =  $order->customer['firstname'];
        $params['CustomerFamilyName'] = $order->customer['lastname'];
        $params['CustomerCountry'] = $order->customer['country']['iso_code_3'];
        $params['CustomerCity'] = $order->customer['city'];
        $params['CustomerZIPCode'] = $order->customer['postcode'];
        $params['CustomerAddress'] = $order->customer['street_address'];
        $params['CustomerPhone'] = $order->customer['telephone'];
		$params['PaymentParametersRequired'] = 1;
        switch (MODULE_PAYMENT_MYPOS_VIRTUAL_PAYMENT_METHOD) {
            case 'All':
                $params['PaymentMethod'] = 3;
                break;
            case 'iDeal':
                $params['PaymentMethod'] = 2;
                break;
            case 'Card Payment':
                $params['PaymentMethod'] = 1;
                break;
        }

        $params['Note'] = 'myPOS Checkout osCommerce Extension';
        $params['CartItems'] = count($order->products) + 1;

        $index = 1;

        /**
         * @var array $item
         */
        foreach($order->products as $item)
        {
            $params['Article_' . $index] = html_entity_decode(strip_tags($item['name']));
            $params['Quantity_' . $index] = number_format($item['qty'], 0, '.', '');
            $params['Price_' . $index] = number_format($item['final_price'], 2, '.', '');
            $params['Amount_' . $index] = number_format($item['final_price'] * $item['qty'], 2, '.', '');
            $params['Currency_' . $index] = $order->info['currency'];

            $index++;
        }

        if (!empty($order->info['tax_groups'])) {
            foreach ($order->info['tax_groups'] as $name => $amount) {
                if ($amount != 0) {
                    $params['Article_' . $index] = $name;
                    $params['Quantity_' . $index] = number_format(1, 0, '.', '');
                    $params['Price_' . $index] = number_format($amount, 2, '.', '');
                    $params['Amount_' . $index] = number_format($amount, 2, '.', '');
                    $params['Currency_' . $index] = $order->info['currency'];
                    $params['CartItems']++;
                    $index++;
                }
            }
        }

        $params['Article_' . $index] = $order->info['shipping_method'];
        $params['Quantity_' . $index] = number_format(1, 0, '.', '');
        $params['Price_' . $index] = number_format($order->info['shipping_cost'], 2, '.', '');
        $params['Amount_' . $index] = number_format($order->info['shipping_cost'], 2, '.', '');
        $params['Currency_' . $index] = $order->info['currency'];

        $params['Signature'] = $this->create_signature($params);

        $process_button_string = '';

        foreach ($params as $key => $value) {
            $process_button_string .= tep_draw_hidden_field($key, $value);
        }

        $payment_mypos_name = MODULE_PAYMENT_MYPOS_VIRTUAL_TEXT_PUBLIC_TITLE
            . ' <input type="button" onclick="location.href=\\\'' . tep_href_link('ext/modules/payment/mypos_virtual/mypos_check_payment_status.php?order_id=' . $order_id) . '\\\';" value="Check Payment Status " />';

        tep_db_query("update " . TABLE_ORDERS . " set payment_method = '" . $payment_mypos_name . "', last_modified = now() where orders_id = '" . $order_id . "'");

        return $process_button_string;
    }

    public function before_process() {
        /**
         * This is where you will implement any payment verification.
         * Called by checkout_process.php before order is finalised
         */

        global $order, $order_totals, $sendto, $billto, $languages_id, $payment, $currencies, $cart, $cart_myPOS_Virtual_ID, $$payment, $HTTP_GET_VARS, $HTTP_POST_VARS, $messageStack, $customer_id;

        $result = $this->is_valid_signature($HTTP_POST_VARS);

        if ($result) {
            if ($HTTP_POST_VARS['IPCmethod'] != 'IPCPurchaseOK') {
                tep_redirect(tep_href_link(FILENAME_SHOPPING_CART));
            }
        } else {
            tep_redirect(tep_href_link(FILENAME_SHOPPING_CART));
        }

        $order_id = substr($cart_myPOS_Virtual_ID, strpos($cart_myPOS_Virtual_ID, '-')+1);

        $check_query = tep_db_query("select orders_status from " . TABLE_ORDERS . " where orders_id = '" . (int)$order_id . "'");

        if (!tep_db_num_rows($check_query) || ($order_id != $HTTP_POST_VARS['OrderID'])) {
            tep_redirect(tep_href_link(FILENAME_SHOPPING_CART));
        }

        $check = tep_db_fetch_array($check_query);

        $new_order_status = 2;

        tep_db_query("update " . TABLE_ORDERS . " set orders_status = '" . (int)$new_order_status . "', last_modified = now() where orders_id = '" . (int)$order_id . "'");

        $sql_data_array = array('orders_id' => $order_id,
            'orders_status_id' => (int)$new_order_status,
            'date_added' => 'now()',
            'customer_notified' => (SEND_EMAILS == 'true') ? '1' : '0',
            'comments' => $order->info['comments']);

        tep_db_perform(TABLE_ORDERS_STATUS_HISTORY, $sql_data_array);

// initialized for the email confirmation
        $products_ordered = '';
        $subtotal = 0;
        $total_tax = 0;

        for ($i=0, $n=sizeof($order->products); $i<$n; $i++) {
// Stock Update - Joao Correia
            if (STOCK_LIMITED == 'true') {
                if (DOWNLOAD_ENABLED == 'true') {
                    $stock_query_raw = "SELECT products_quantity, pad.products_attributes_filename
                                FROM " . TABLE_PRODUCTS . " p
                                LEFT JOIN " . TABLE_PRODUCTS_ATTRIBUTES . " pa
                                ON p.products_id=pa.products_id
                                LEFT JOIN " . TABLE_PRODUCTS_ATTRIBUTES_DOWNLOAD . " pad
                                ON pa.products_attributes_id=pad.products_attributes_id
                                WHERE p.products_id = '" . tep_get_prid($order->products[$i]['id']) . "'";
// Will work with only one option for downloadable products
// otherwise, we have to build the query dynamically with a loop
                    $products_attributes = $order->products[$i]['attributes'];
                    if (is_array($products_attributes)) {
                        $stock_query_raw .= " AND pa.options_id = '" . $products_attributes[0]['option_id'] . "' AND pa.options_values_id = '" . $products_attributes[0]['value_id'] . "'";
                    }
                    $stock_query = tep_db_query($stock_query_raw);
                } else {
                    $stock_query = tep_db_query("select products_quantity from " . TABLE_PRODUCTS . " where products_id = '" . tep_get_prid($order->products[$i]['id']) . "'");
                }
                if (tep_db_num_rows($stock_query) > 0) {
                    $stock_values = tep_db_fetch_array($stock_query);
// do not decrement quantities if products_attributes_filename exists
                    if ((DOWNLOAD_ENABLED != 'true') || (!$stock_values['products_attributes_filename'])) {
                        $stock_left = $stock_values['products_quantity'] - $order->products[$i]['qty'];
                    } else {
                        $stock_left = $stock_values['products_quantity'];
                    }
                    tep_db_query("update " . TABLE_PRODUCTS . " set products_quantity = '" . $stock_left . "' where products_id = '" . tep_get_prid($order->products[$i]['id']) . "'");
                    if ( ($stock_left < 1) && (STOCK_ALLOW_CHECKOUT == 'false') ) {
                        tep_db_query("update " . TABLE_PRODUCTS . " set products_status = '0' where products_id = '" . tep_get_prid($order->products[$i]['id']) . "'");
                    }
                }
            }

// Update products_ordered (for bestsellers list)
            tep_db_query("update " . TABLE_PRODUCTS . " set products_ordered = products_ordered + " . sprintf('%d', $order->products[$i]['qty']) . " where products_id = '" . tep_get_prid($order->products[$i]['id']) . "'");

//------insert customer choosen option to order--------
            $attributes_exist = '0';
            $products_ordered_attributes = '';
            if (isset($order->products[$i]['attributes'])) {
                $attributes_exist = '1';
                for ($j=0, $n2=sizeof($order->products[$i]['attributes']); $j<$n2; $j++) {
                    if (DOWNLOAD_ENABLED == 'true') {
                        $attributes_query = "select popt.products_options_name, poval.products_options_values_name, pa.options_values_price, pa.price_prefix, pad.products_attributes_maxdays, pad.products_attributes_maxcount , pad.products_attributes_filename
                                   from " . TABLE_PRODUCTS_OPTIONS . " popt, " . TABLE_PRODUCTS_OPTIONS_VALUES . " poval, " . TABLE_PRODUCTS_ATTRIBUTES . " pa
                                   left join " . TABLE_PRODUCTS_ATTRIBUTES_DOWNLOAD . " pad
                                   on pa.products_attributes_id=pad.products_attributes_id
                                   where pa.products_id = '" . $order->products[$i]['id'] . "'
                                   and pa.options_id = '" . $order->products[$i]['attributes'][$j]['option_id'] . "'
                                   and pa.options_id = popt.products_options_id
                                   and pa.options_values_id = '" . $order->products[$i]['attributes'][$j]['value_id'] . "'
                                   and pa.options_values_id = poval.products_options_values_id
                                   and popt.language_id = '" . $languages_id . "'
                                   and poval.language_id = '" . $languages_id . "'";
                        $attributes = tep_db_query($attributes_query);
                    } else {
                        $attributes = tep_db_query("select popt.products_options_name, poval.products_options_values_name, pa.options_values_price, pa.price_prefix from " . TABLE_PRODUCTS_OPTIONS . " popt, " . TABLE_PRODUCTS_OPTIONS_VALUES . " poval, " . TABLE_PRODUCTS_ATTRIBUTES . " pa where pa.products_id = '" . $order->products[$i]['id'] . "' and pa.options_id = '" . $order->products[$i]['attributes'][$j]['option_id'] . "' and pa.options_id = popt.products_options_id and pa.options_values_id = '" . $order->products[$i]['attributes'][$j]['value_id'] . "' and pa.options_values_id = poval.products_options_values_id and popt.language_id = '" . $languages_id . "' and poval.language_id = '" . $languages_id . "'");
                    }
                    $attributes_values = tep_db_fetch_array($attributes);

                    $products_ordered_attributes .= "\n\t" . $attributes_values['products_options_name'] . ' ' . $attributes_values['products_options_values_name'];
                }
            }
//------insert customer choosen option eof ----
            $total_weight += ($order->products[$i]['qty'] * $order->products[$i]['weight']);
            $total_tax += tep_calculate_tax($total_products_price, $products_tax) * $order->products[$i]['qty'];
            $total_cost += $total_products_price;

            $products_ordered .= $order->products[$i]['qty'] . ' x ' . $order->products[$i]['name'] . ' (' . $order->products[$i]['model'] . ') = ' . $currencies->display_price($order->products[$i]['final_price'], $order->products[$i]['tax'], $order->products[$i]['qty']) . $products_ordered_attributes . "\n";
        }

// lets start with the email confirmation
        $email_order = STORE_NAME . "\n" .
            EMAIL_SEPARATOR . "\n" .
            EMAIL_TEXT_ORDER_NUMBER . ' ' . $order_id . "\n" .
            EMAIL_TEXT_INVOICE_URL . ' ' . tep_href_link(FILENAME_ACCOUNT_HISTORY_INFO, 'order_id=' . $order_id, 'SSL', false) . "\n" .
            EMAIL_TEXT_DATE_ORDERED . ' ' . strftime(DATE_FORMAT_LONG) . "\n\n";
        if ($order->info['comments']) {
            $email_order .= tep_db_output($order->info['comments']) . "\n\n";
        }
        $email_order .= EMAIL_TEXT_PRODUCTS . "\n" .
            EMAIL_SEPARATOR . "\n" .
            $products_ordered .
            EMAIL_SEPARATOR . "\n";

        for ($i=0, $n=sizeof($order_totals); $i<$n; $i++) {
            $email_order .= strip_tags($order_totals[$i]['title']) . ' ' . strip_tags($order_totals[$i]['text']) . "\n";
        }

        if ($order->content_type != 'virtual') {
            $email_order .= "\n" . EMAIL_TEXT_DELIVERY_ADDRESS . "\n" .
                EMAIL_SEPARATOR . "\n" .
                tep_address_label($customer_id, $sendto, 0, '', "\n") . "\n";
        }

        $email_order .= "\n" . EMAIL_TEXT_BILLING_ADDRESS . "\n" .
            EMAIL_SEPARATOR . "\n" .
            tep_address_label($customer_id, $billto, 0, '', "\n") . "\n\n";

        if (is_object($$payment)) {
            $email_order .= EMAIL_TEXT_PAYMENT_METHOD . "\n" .
                EMAIL_SEPARATOR . "\n";
            $payment_class = $$payment;
            $email_order .= $payment_class->title . "\n\n";
            if ($payment_class->email_footer) {
                $email_order .= $payment_class->email_footer . "\n\n";
            }
        }

        tep_mail($order->customer['firstname'] . ' ' . $order->customer['lastname'], $order->customer['email_address'], EMAIL_TEXT_SUBJECT, $email_order, STORE_OWNER, STORE_OWNER_EMAIL_ADDRESS);

// send emails to other people
        if (SEND_EXTRA_ORDER_EMAILS_TO != '') {
            tep_mail('', SEND_EXTRA_ORDER_EMAILS_TO, EMAIL_TEXT_SUBJECT, $email_order, STORE_OWNER, STORE_OWNER_EMAIL_ADDRESS);
        }

// load the after_process function from the payment modules
        $this->after_process();

        $cart->reset(true);

// unregister session variables used during checkout
        tep_session_unregister('sendto');
        tep_session_unregister('billto');
        tep_session_unregister('shipping');
        tep_session_unregister('payment');
        tep_session_unregister('comments');

        tep_session_unregister('cart_PayPal_Standard_ID');

        tep_redirect(tep_href_link(FILENAME_CHECKOUT_SUCCESS, '', 'SSL'));

        return false;
    }

    public function after_process() {
        /**
         * Here you may implement any post proessing of the payment/order after the order has been finalised. At this point you now have a reference to the created osCommerce order id and you would typically update any custom database tables you may have for your module. You most probably don't need to implement anything here.
         * Called by checkout_process.php after order is finalised
         */
        return false;
    }

    public function get_error() {
        /**
         * For more advanced error handling. When your module logic returns any errors you will redirect to checkout_payment.php with some error information. When implemented corretly, this function can be used to genereate the proper error texts for particular errors. The redirect must be formatted like this:
         * tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error=' . $this->code.'&error='.urlencode('some error'), 'NONSSL', true, false));
         * Called by checkout_payment.php
         */
        //TODO
        return false;
    }

    public function check() {
        /**
         * Standard functionlity for osCommerce to see if the module is installed.
         */
        if (!isset($this->_check)) {
            $check_query = tep_db_query("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'MODULE_PAYMENT_MYPOS_VIRTUAL_STATUS'");
            $this->_check = tep_db_num_rows($check_query);
        }

        return $this->_check;
    }

    public function install($parameter = null) {
        /**
         * This is where you define module's configurations (displayed in admin).
         */
        $params = $this->getParams();

        if (isset($parameter)) {
            if (isset($params[$parameter])) {
                $params = array($parameter => $params[$parameter]);
            } else {
                $params = array();
            }
        }

        foreach ($params as $key => $data) {
            $this->addConfig($key, $data);
        }
    }

    public function remove() {
        /**
         * Standard functionality to uninstall the module.
         */
        tep_db_query("delete from " . TABLE_CONFIGURATION . " where configuration_key in ('" . implode("', '", $this->keys()) . "')");
    }

    public function addConfig($key, $data)
    {
            $sql_data_array = array(
                'configuration_title' => $data['title'],
                'configuration_key' => $key,
                'configuration_value' => (isset($data['value']) ? $data['value'] : ''),
                'configuration_description' => $data['desc'],
                'configuration_group_id' => '6',
                'sort_order' => '0',
                'date_added' => 'now()'
            );

            if (isset($data['set_func'])) {
                $sql_data_array['set_function'] = $data['set_func'];
            }

            if (isset($data['use_func'])) {
                $sql_data_array['use_function'] = $data['use_func'];
            }

            tep_db_perform(TABLE_CONFIGURATION, $sql_data_array);
        }

    public function editConfig($key, $data)
    {
        $sql_data_array = array(
            'configuration_title' => $data['title'],
            'configuration_key' => $key,
            'configuration_value' => (isset($data['value']) ? $data['value'] : ''),
            'configuration_description' => $data['desc'],
            'configuration_group_id' => '6',
            'sort_order' => '0',
            'date_added' => 'now()'
        );

        if (isset($data['set_func'])) {
            $sql_data_array['set_function'] = $data['set_func'];
        }

        if (isset($data['use_func'])) {
            $sql_data_array['use_function'] = $data['use_func'];
        }

        tep_db_perform(TABLE_CONFIGURATION, $sql_data_array, 'update', '`configuration_key`="' . $key . '"');
    }

    public function keys() {
        /**
         * This array must include all the configuration setting keys defined in your install() function.
         */
        $keys = array_keys($this->getParams());

        if ($this->check()) {
            foreach ($keys as $key) {
                if (!defined($key)) {
                    $this->install($key);
                }
            }
        }

        return $keys;
    }

    public function getParams() {
        $params = array(
            'MODULE_PAYMENT_MYPOS_VIRTUAL_STATUS' => array(
                'title' => 'Enable myPOS Checkout',
                'desc' => 'Enable myPOS Checkout Payment Method',
                'value' => 'True',
                'set_func' => 'tep_cfg_select_option(array(\'True\', \'False\'), '
            ),
            'MODULE_PAYMENT_MYPOS_VIRTUAL_PAYMENT_METHOD' => array(
                'title' => 'Payment Method',
                'desc' => 'Enable myPOS Checkout Payment Method',
                'value' => 'All',
                'set_func' => 'tep_cfg_select_option(array(\'All\', \'Card Payment\', \'iDeal\'), '
            ),
            'MODULE_PAYMENT_MYPOS_VIRTUAL_TEST_MODE' => array(
                'title' => 'Test Mode',
                'desc' => 'In order to test integration with myPOS Checkout you need to use the testing data available in your online banking at www.mypos.com > menu Online > Shopping carts > osCommerce.',
                'value' => 'True',
                'set_func' => 'tep_cfg_select_option(array(\'True\', \'False\'), '
            ),
            'MODULE_PAYMENT_MYPOS_VIRTUAL_TEST_PREFIX' => array(
                'title' => 'Test Prefix',
                'desc' => 'Only used in test mode.',
                'value' => uniqid() . '_',
                'set_func' => 'tep_cfg_select_option(array(MODULE_PAYMENT_MYPOS_VIRTUAL_TEST_PREFIX), '
            ),
            'MODULE_PAYMENT_MYPOS_VIRTUAL_SORT_ORDER' => array(
                'title' => 'Sort order of display',
                'desc' => 'Sort order of display. Lowest is displayed first.',
                'value' => '0'
            ),
            'MODULE_PAYMENT_MYPOS_VIRTUAL_CONFIGURATION_PACK' => array(
                'title' => 'Configuration Pack',
                'desc' => 'Paste in your configuration pack in the field and press update. This will override your plugin settings. <br /><a href="https://developers.mypos.com/en/doc/online_payments/v1_4/5-store-management" target="blank">Generate New Pack</a>',
            ),
            'MODULE_PAYMENT_MYPOS_VIRTUAL_SITE_ID' => array(
                'title' => 'Store ID',
                'desc' => 'Store ID is given when you add a new online store. It could be reviewed in your online banking at www.mypos.com > menu Online > Online stores.'
            ),
            'MODULE_PAYMENT_MYPOS_VIRTUAL_WALLET_NR' => array(
                'title' => 'Client Number',
                'desc' => 'You can view your myPOS Client number in your online banking at www.mypos.com'
            ),
            'MODULE_PAYMENT_MYPOS_VIRTUAL_PRIVATE_KEY' => array(
                'title' => 'Private Key',
                'desc' => 'The Private Key for your store is generated in your online banking at www.mypos.com > menu Online > Online stores > Keys.',
            ),
            'MODULE_PAYMENT_MYPOS_VIRTUAL_PUBLIC_CERTIFICATE' => array(
                'title' => 'myPOS Public Certificate',
                'desc' => 'The myPOS Public Certificate is available for download in your online banking at www.mypos.com > menu Online > Online stores > Keys.',
            ),
            'MODULE_PAYMENT_MYPOS_VIRTUAL_KEYINDEX' => array(
                'title' => 'Key Index',
                'desc' => 'The myPOS Public Certificate is available for download in your online banking at www.mypos.com > menu Online > Online stores > Keys.'
            ),
            'MODULE_PAYMENT_MYPOS_VIRTUAL_DEVELOPER_URL' => array(
                'title' => 'Developer URL',
                'value' => 'https://www.mypos.com/vmp/checkout-test'
            ),
            'MODULE_PAYMENT_MYPOS_VIRTUAL_PRODUCTION_URL' => array(
                'title' => 'Production URL',
                'value' => 'https://www.mypos.com/vmp/checkout',
            ),
            'MODULE_PAYMENT_MYPOS_VIRTUAL_CRON_URL' => array(
                'title' => 'Cron URL',
                'desc' => 'Check your last Pending orders for myPOS Payment',
                'value' => tep_catalog_href_link('ext/modules/payment/mypos_virtual/mypos_cron.php'),
                'set_func' => 'tep_cfg_select_option(array(MODULE_PAYMENT_MYPOS_VIRTUAL_CRON_URL), '
            ),
        );

        return $params;
    }

    public function sendDebugEmail($response = '') {
        global $HTTP_POST_VARS, $HTTP_GET_VARS;
    }

    public function getValuesFromMultidimensionalArray($array, $allValues = array()) {
        foreach ($array as $item) {
            if (is_array($item)) {
                $allValues = $this->getValuesFromMultidimensionalArray($item, $allValues);
            } else {
                $allValues[] = $item;
            }
        }

        return $allValues;
    }

    private function create_signature($post)
    {
        $this->private_key = trim(str_replace('-----BEGIN RSA PRIVATE KEY-----', '', $this->private_key));
        $this->private_key = trim(str_replace('-----END RSA PRIVATE KEY-----',   '', $this->private_key));

        $this->private_key = '-----BEGIN RSA PRIVATE KEY-----' . "\n" . $this->private_key;

        for ($i = 96; $i < strlen($this->private_key); $i += 65) {
            $this->private_key[$i] = "\n";
        }

        $this->private_key .= "\n" . '-----END RSA PRIVATE KEY-----';
        
        $concData = base64_encode(implode('-', $post));
        $privKeyObj = openssl_get_privatekey($this->private_key);
        openssl_sign($concData, $signature, $privKeyObj, OPENSSL_ALGO_SHA256);
        return base64_encode($signature);
    }

    public function is_valid_signature($post)
    {
        $this->public_certificate = trim(str_replace('-----BEGIN CERTIFICATE-----', '', $this->public_certificate));
        $this->public_certificate = trim(str_replace('-----END CERTIFICATE-----',   '', $this->public_certificate));

        $this->public_certificate = '-----BEGIN CERTIFICATE-----' . "\n" . $this->public_certificate;

        for ($i = 92; $i < strlen($this->public_certificate); $i += 65) {
            $this->public_certificate[$i] = "\n";
        }

        $this->public_certificate .= "\n" . '-----END CERTIFICATE-----';

        // Save signature
        $signature = $post['Signature'];

        // Remove signature from POST data array
        unset($post['Signature']);

        // Concatenate all values
        $concData = base64_encode(implode('-', $post));

        // Extract public key from certificate
        $pubKeyId = openssl_get_publickey($this->public_certificate);

        // Verify signature
        $result = openssl_verify($concData, base64_decode($signature), $pubKeyId, OPENSSL_ALGO_SHA256);

        //Free key resource
        openssl_free_key($pubKeyId);

        if ($result == 1)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    public function check_order_payment_status($order_id)
    {
        $order_id = tep_db_input($order_id);

        $order_query = tep_db_query("select orders_status from " . TABLE_ORDERS . " where orders_id = '" . $order_id . "'");

        if ( tep_db_num_rows($order_query) === 1 ) {
            $order = tep_db_fetch_array($order_query);
            if ($order['orders_status'] == 1) {
            //open connection
            $ch = curl_init($this->url);

            $data = array();

            $data = array_merge($data, $this->_defaultMethodProperties);

            $data['IPCmethod'] = 'IPCGetPaymentStatus';
            $data['OrderID'] = $order_id;

            $data['OutputFormat'] = 'json';

            $data['Signature'] = $this->create_signature($data);
            $url =  $this->form_action_url = MODULE_PAYMENT_MYPOS_VIRTUAL_TEST_MODE == 'True' ? MODULE_PAYMENT_MYPOS_VIRTUAL_DEVELOPER_URL : MODULE_PAYMENT_MYPOS_VIRTUAL_PRODUCTION_URL;
            //set the url, number of POST vars, POST data
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($ch, CURLOPT_POST,1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION ,1);
            curl_setopt($ch, CURLOPT_HEADER ,0); // DO NOT RETURN HTTP HEADERS
            curl_setopt($ch, CURLOPT_RETURNTRANSFER ,1); // RETURN THE CONTENTS OF THE CALL
            curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 120); // Timeout on connect (2 minutes)

            //execute post
            $result = curl_exec($ch);
            curl_close($ch);
            $post = json_decode($result, true);

            if ($this->is_valid_signature($post) && array_key_exists('PaymentStatus', $post))
            {

                switch ($post['PaymentStatus']) {
                    case 1:
                        $new_order_status = 2;

                        $comment_status = 'myPOS authorized order payment. Transaction ID: ' . $post['IPC_Trnref'] . '; ';

                        tep_db_query("update " . TABLE_ORDERS . " set orders_status = '" . (int)$new_order_status . "', last_modified = now() where orders_id = '" . (int)$order_id . "'");

                        $sql_data_array = array('orders_id' => $order_id,
                            'orders_status_id' => 2,
                            'date_added' => 'now()',
                            'customer_notified' => '0',
                            'comments' => '[' . tep_output_string_protected($comment_status) . ']');

                        tep_db_perform(TABLE_ORDERS_STATUS_HISTORY, $sql_data_array);
                        break;
                    case 4:
                    case 3:
                        $new_order_status = 1;

                        $comment_status = 'myPOS authorized payment as error';

                        tep_db_query("update " . TABLE_ORDERS . " set orders_status = '" . (int)$new_order_status . "', last_modified = now() where orders_id = '" . (int)$order_id . "'");


                        $sql_data_array = array('orders_id' => (int)$order['orders_id'],
                            'orders_status_id' => 1,
                            'date_added' => 'now()',
                            'customer_notified' => '0',
                            'comments' => '[' . tep_output_string_protected($comment_status) . ']');
                        tep_db_perform(TABLE_ORDERS_STATUS_HISTORY, $sql_data_array);
                        break;
                }
            }

            }
        }
    }
}
